import { key, MODIFIER } from './key';
import { isArray, isObject } from './types';
function merge_value_to_array(array, _value) {
    if (typeof _value === 'string') {
        const [val, mod] = key(_value);
        if (mod === MODIFIER.REMOVE) {
            const i = array.indexOf(val);
            array.splice(i, 1);
        }
        else {
            array.push(val);
        }
    }
    else {
        array.push(_value);
    }
}
function merge_array(array, _value) {
    const val = isArray(_value) ? _value : [_value];
    for (const v of val) {
        merge_value_to_array(array, v);
    }
}
function merge_value_to_object(object, _key, _value) {
    const [k, mod] = key(_key);
    if (mod === MODIFIER.REMOVE) {
        object[k] = undefined;
    }
    else if (mod === MODIFIER.ADD || object[k] == null) {
        object[k] = _value;
    }
    else {
        if (isArray(object[k])) {
            merge_array(object[k], _value);
        }
        else if (isObject(object[k])) {
            merge_object(object[k], _value);
        }
        else {
            object[k] = _value;
        }
    }
}
function merge_object(object, _value) {
    if (!isObject(_value))
        return;
    for (const k in _value) {
        if (Object.prototype.hasOwnProperty.call(_value, k)) {
            merge_value_to_object(object, k, _value[k]);
        }
    }
}
export function merge(json, value) {
    if (!isObject(json))
        return;
    merge_object(json, value);
}
